<?php
// ******************************* kh_mod RANDOM PICTURE ******************************** //
/**
 * Addon 'Random Picture' for kh_mod 0.4.9 ONLY, installation see the file 'readme.txt'.
 */

// GALLERY URL WITH THIS SCRIPT, e.g. http://www.yourdomain.com/gallery/
$url_gallery = 'xxx';

// ALLOWED URL TO ACCESS THIS SCRIPT, e.g. http://www.yourrandom.com/
$url_access  = 'xxx';

// ACCESS TO THIS SCRIPT VIA SSI (SERVER SIDE INCLUDES)
$ssi_access  = false;	// true or false

// ********************************** End of Settings *********************************** //

// NO ERROR REPORTING
error_reporting(0);

// GET VALUES FOR HTTP_REFERER CHECK
$url_referer = filter_input(INPUT_SERVER, 'HTTP_REFERER');
$len_access	 = strlen($url_access);

// ACCESS PER SSI?
if ($ssi_access) {
    // doesn't it run on command line?
    if (empty($_SERVER['argc']))                                    exit();
}
// ALLOWD URL SPECIFIED?
elseif ($len_access > 0) {
    // http referer different to specified url?
    if (strncasecmp($url_referer, $url_access, $len_access) !== 0)  exit();
}

// GET PARAMETERS
$imageID    = isset($_GET['iID']) ? (int)$_GET['iID']:false;         // image to display
$remoteID   = isset($_GET['rID']) ? (int)$_GET['rID']:false;         // id for session 'remoteID'
$img_output = isset($_GET['o'])   ? substr($_GET['o'], 0, 5):false;  // 'scale' or 'crop' original picture or display 'thumb'
$img_width  = isset($_GET['w'])   ? min((int)$_GET['w'], 1024):150;  // max. width to scale or exact value to crop random picture
$img_height = isset($_GET['h'])   ? min((int)$_GET['h'], 1024):150;  // max. height to scale or exact value to crop random picture
$img_title  = empty($_GET['t'])   ? '':substr($_GET['t'], 0,16);     // 'show', 'utf8dec' or 'hide', display title or don't do it
$url_target = empty($_GET['a'])   ? '_top':substr($_GET['a'], 0,16); // target window of the random picture link
$url_iframe = (filter_input(INPUT_GET, 'f') == 1) ? true:false;      // display the random picture in an iframe
$html_code  = ($remoteID || $imageID) ? false:true;                  // output html tags for random image

// DEFINE SCRIPT CONSTANS
define('ROOT_FOLDER',  __DIR__);
define('GALLERY_ID', 'mg2'. (string)crc32(ROOT_FOLDER));
define('INC_FOLDER',  ROOT_FOLDER .'/includes/');
define('DATA_FOLDER', ROOT_FOLDER .'/data/');

// INSTALL AUTOLOADER
require_once ROOT_FOLDER .'/vendor/autoload.php';

// MAKE RANDOM OBJECT
$random = new MG2Random();

// SET IMAGE ID
if (! $random->setImageID($imageID, $remoteID)) exit();

// SET IMAGE FILEPATH
if (! $random->setImagePath($img_output))       exit();

// *************************************** OUTPUT *************************************** //
// OUTPUT HTML ONLY
if ($html_code) {
	$random->outputHTMLTag($img_width, $img_height, $img_output, $img_title, $url_gallery, $url_target, $url_iframe);
}
// OUTPUT SCALED IMAGE
elseif ($img_output === 'scale') {
	$random->outputScaledImage($img_width, $img_height);
}
// OUTPUT CROPED IMAGE
elseif ($img_output === 'crop') {
	$random->outputCropedImage($img_width, $img_height);
}
// OUTPUT THUMBNAIL ONLY
elseif ($img_output === 'thumb') {
	$random->outputThumbnail();
}
// *********************************** END OF OUTPUT ************************************ //

// MG2RANDOM CLASS
class MG2Random
{
    private $mg2;           // @var MG2Gallery object
    private $imageID;       // @var int of image id
    private $imageFilepath; // @var string of path to image root
    private $folderpath;    // @var string of invalid folders

    public function __construct()
    {
        // INSTANTIATE GALLERY CLASS
        $mg2 = new MG2Gallery();

        // READ DATABASE FOR GALLERY
        $mg2->readDB();

        // SAVE AS MEMBER ATTRIBUT
        $this->mg2 = $mg2;

        // INIT INVALID FOLDERS
        $this->folderpath = [0=>true];  // 0, first invalid id
    }

    /**
     * SET IMAGE ID
     *
     * @param int|false $imageID
     * @param int|false $remoteID
     */
    public function setImageID($imageID, $remoteID)
    {
        // CHECK REQUESTED IMAGE ID
        if ($imageID) {
            $this->imageID = $this->checkImageID($imageID) ?
                $imageID
                :
                false;
        }
        // GET A VALID RANDOM IMAGE ID
        else {
            $this->imageID = $this->getRandomImageID($remoteID);
        }

        return $this->imageID ? true:false;
    }

    /**
     * SET FILE PATH OF RANDOM IMAGE
     *
     * @param string $imgOutput
     * @return boolean
     */
    public function setImagePath($imgOutput)
    {
        $this->imageFilepath = $this->mg2->getFilePathByID(
            $this->imageID,                                 // image id
            $imgOutput === 'thumb' ? 'thumb':'item'         // use thumb or image
        );

        return is_readable($this->imageFilepath) ?
            true
            :
            false;
    }

    /**
     * GET VALID RANDOM IMAGE ID
     *
     * @param int|false $remoteID
     * @return int
     */
    private function getRandomImageID($remoteID)
    {
        // SESSION START
        session_start();

        // INIT IMAGE ID
        $imageID = false;

        // LOOP ALL IMAGE RECORDS UPTO A VALID IMAGE
        while (! empty($this->mg2->all_items))
        {
            $randomID = array_rand($this->mg2->all_items);

            // CHECK FOLDER PATH FOR RANDOM PICTURE
            if (! $this->checkImageID($randomID))
            {
                unset($this->mg2->all_items[$randomID]); continue;  // delete invalid image record
            }

            // SET IMAGE ID AND SESSION
            $imageID = $randomID;
            if ($remoteID !== false) {
                $_SESSION[GALLERY_ID]['remoteID'][$remoteID] = $randomID;
            }

            // VALID IMAGE ID
            break;
        }

        return $imageID;
    }

    /**
     * CHECK IMAGE ID AGAINST PARENT FOLDERS
     * (todo: this method could/should possibly be realised by method
     *  MG2gallery::checkGalleryPassword)
     *
     * @param int|false $imageID
     * @return boolean
     */
    private function checkImageID($imageID)
    {
        $currentID   = 1;       // check upto root folder
        $folderCheck = false;   // folder path check

        // GET PARENT FOLDER ID OF AN IMAGE
        $folderID = (int)($this->mg2->all_items[$imageID][1] ?? 0);

        if ( isset($this->folderpath[$folderID])                             // already checked as invalid folder
             || !isset($this->mg2->all_folders[$folderID])                   // parent folder does not exist
             || (int)$this->mg2->all_folders[$folderID][11]                  // no default parent folder
             || !$this->checkFolderPassword($folderID)                       // password protected folder
             || !$this->mg2->getContentTypeByID($imageID, 'image'))
        {
            return $folderCheck;
        }

        // SAVE START FOLDER ID
        $this->folderpath[$folderID] = true;

        // CHECK FOLDER PATH UPTO ROOT
        while ($folderID > 0) {
            if ($folderID === $currentID)	{ $folderCheck = true; break; }  // parent folder valid

            $folderID = (int)$this->mg2->all_folders[$folderID][1];          // get parent folder id
            if (isset($this->folderpath[$folderID]))               break;    // circle data or damaged folder record
            $this->folderpath[$folderID] = true;                             // save parent folder id
            if (empty($this->mg2->all_folders[$folderID]))         break;    // parent folder not exists

            // SUB FOLDER PASSWORD PROTECTION
            if ((~$this->mg2->extendedset & 1)
                && $this->checkFolderPassword($folderID) === false)
            {
                break;   // password protected
            }
        }
        return $folderCheck;
    }

    /**
     * CHECK FOLDER PASSWORD
     *
     * @param int $folderID
     * @return boolean
     */
    private function checkFolderPassword($folderID)
    {
        $pwdAccess = true;
        $folderpwd = trim($this->mg2->all_folders[$folderID][8] ?? '');

        do {
            // NO FOLDER PASSWORD
            if (! $folderpwd) {
                break;
            }

            // FOLDER PASSWORD OK
            if (! empty($_SESSION[GALLERY_ID]['fPwdHash'][$folderID])
                && $_SESSION[GALLERY_ID]['fPwdHash'][$folderID] === $folderpwd)
            {
                break;
            }
            
            // ADMIN MODE ACTIVE
            if (! empty($_SESSION[GALLERY_ID]['adminmode'])
                && $_SESSION[GALLERY_ID]['adminmode'] === $this->mg2->adminpwd)
            {
                break;
            }

            $pwdAccess = false;
        }
        while(0);

        return $pwdAccess;
    }

    /**
     * OUTPUT HTML TAGS FOR RANDOM IMAGE
     *
     * @param int $randWidth
     * @param int $randHeight
     * @param string $img_output
     * @param string $img_title
     * @param string $url_gallery
     * @param string $url_target
     * @param boolean $url_iframe
     * @return void
     */
    public function outputHTMLTag($randWidth, $randHeight, $img_output, $img_title, $url_gallery, $url_target, $url_iframe)
    {
        // SOURCEIMAGE
        if (!$imageInfo = getimagesize($this->imageFilepath)) exit;

        // CALCULATE THE SMALLER IMAGE SIDE
        $imageWidth  = max($imageInfo[0], 1);   // to avoid divison by zero
        $imageHeight = max($imageInfo[1], 1);   // to avoid divison by zero
        if ($img_output === 'scale') {
            if ($imageWidth < $imageHeight) {
                $randWidth  = (int)round($imageWidth/$imageHeight * $randHeight);
            } else {
                $randHeight = (int)round($imageHeight/$imageWidth * $randWidth);
            }
        }
        // GET THE ORIGINAL THUMB SIZE
        elseif ($img_output === 'thumb') {
            $randWidth  = $imageWidth;
            $randHeight = $imageHeight;
        }

        // GET TITLE FOR RANDOM PICTURE
        if ($img_title && ($img_title !== 'hide')) {
            // use filename as picture headline
            $randTitle = htmlspecialchars(($this->mg2->layoutsetting & 8) ?
                $this->mg2->getFilename($this->imageID)
                :
                strip_tags($this->mg2->br2line($this->mg2->all_items[$this->imageID][2])));
            // converts UTF-8 to single-byte ISO-8859-1 text
            if ($img_title === 'utf8dec') {
                $randTitle = $this->mg2->utf8_decode($randTitle);
            }
        }
        else {
            $randTitle = '';
        }

        // URL OF THE GALLERY AND RANDOM SCRIPT
        $url_index  = $url_gallery . $this->mg2->getGalleryLink(['iID'=>$this->imageID]);
        $url_random = $url_gallery . 'random.php';

        // HTML HEADER AND FOOTER
        $header = $footer = '';
        if ($url_iframe) {
            $header = '<!DOCTYPE html>' . "\n"
                     .'<html>' . "\n"
                     .'<head><title>random picture</title></head>' ."\n"
                     .'<body style="margin:0;padding:0;">' . "\n";
            $footer = "\n</body>\n</html>";
        }

        // OUTPUT HTML TAGS
        $random_tag = '<a href="%1$s" target="%2$s">';
        $random_tag.= '<img class="mg2RandomPicture" src="%3$s?iID=%4$d&amp;o=%5$s&amp;w=%6$d&amp;h=%7$d&amp;nc=%9$d" width="%6$d" height="%7$d" border="0" alt="%8$s" title="%8$s" />';
        $random_tag.= '</a>';
        printf($header . $random_tag . $footer,
            $url_index,         // %1 - absolute path to random picture, e.g. http://www.yourdomain.com/gallry/image20.html
            $url_target,        // %2 - target of the random picture link
            $url_random,        // %3 - absolute path to this script, e.g. http://www.yourdomain.com/gallry/random.php
            $this->imageID,     // %4 - id of picture which is to be displayed
            $img_output,        // %5 - 'scale' or 'crop' original picture or display 'thumb'
            $randWidth,         // %6 - max. width for scale or exact value for crop random picture
            $randHeight,        // %7 - max. height for scale or exact value for crop random picture
            $randTitle,         // %8 - 'alt' and 'title' attribute for random picture
            mt_rand(1000,9999)  // %9 - don't use cached pictures
        );
    }

    /**
     * SCALE IMAGE AND OUTPUT
     *
     * @param int $maxWidth
     * @param int $maxHeight
     * @return void
     */
    public function outputScaledImage($maxWidth, $maxHeight)
    {
        // SOURCEIMAGE VALUES
        list($imageWidth,
             $imageHeight,
             $imagecreate,
             $imageformat,
             $imagequality,
             $mimetype
        ) = $this->getSourceImageValues();

        // CALCULATE THE SMALLER IMAGE SIDE
        if ($imageWidth < $imageHeight) {
            $maxWidth  = (int)round($imageWidth/$imageHeight * $maxHeight);
        } else {
            $maxHeight = (int)round($imageHeight/$imageWidth * $maxWidth);
        }

        // CREATE NEW PICTURE
        $imageDest = imagecreatetruecolor($maxWidth, $maxHeight);

        // CREATE SOURCE IMAGE
        $imageSrc  = $imagecreate($this->imageFilepath);

        // RESIZE IMAGE
        imagecopyresampled($imageDest, $imageSrc, 0, 0, 0, 0, $maxWidth, $maxHeight, $imageWidth, $imageHeight);

        // OUTPUT IMAGE
        $this->outputHeader($mimetype);
        $imageformat($imageDest, null, $imagequality);

        // DELETE ACCESS MEMORY
        imagedestroy($imageSrc);
        imagedestroy($imageDest);
    }

    /**
     * @param int $cropWidth
     * @param int $cropHeight
     * @return void
     */
    public function outputCropedImage($cropWidth, $cropHeight)
    {
        // SOURCEIMAGE VALUES
        list($imageWidth,
             $imageHeight,
             $imagecreate,
             $imageformat,
             $imagequality,
             $mimetype
        ) = $this->getSourceImageValues();

        // CALCULATE CROP AREA
        $cropArea   = 0.9;                                                         // maximal area for width and height
        $cropBorder = (1 - $cropArea)/2;                                           // minimal border for crop image

        $ratioW = $cropArea * ($imageWidth/$cropWidth);                            // original to crop width
        $ratioH = $cropArea * ($imageHeight/$cropHeight);                          // original to crop height

        if ($ratioW < $ratioH) {
            $sourceX     = (int)floor($imageWidth * $cropBorder);
            $imageWidth  = (int)round($imageWidth * $cropArea);

            $sourceY     = (int)floor(($imageHeight - ($cropHeight * $ratioW))/3); // vertical one third from top
            $imageHeight = (int)round($cropHeight * $ratioW);
        }
        else {
            $sourceY     = (int)floor($imageHeight * $cropBorder);
            $imageHeight = (int)round($imageHeight * $cropArea);

            $sourceX     = (int)floor(($imageWidth - ($cropWidth * $ratioH))/2);   // horizontal to center
            $imageWidth  = (int)round($cropWidth * $ratioH);
        }

        // CREATE NEW PICTURE
        $imageDest = imagecreatetruecolor($cropWidth, $cropHeight);

        // CREATE SOURCE IMAGE
        $imageSrc  = $imagecreate($this->imageFilepath);

        // RESIZE IMAGE
        imagecopyresampled($imageDest, $imageSrc, 0, 0, $sourceX, $sourceY, $cropWidth, $cropHeight, $imageWidth, $imageHeight);

        // OUTPUT IMAGE
        $this->outputHeader($mimetype);
        $imageformat($imageDest, null, $imagequality);

        // DELETE ACCESS MEMORY
        imagedestroy($imageSrc);
        imagedestroy($imageDest);
    }

    /**
     * @return array|void
     */
    private function getSourceImageValues()
    {
        if (!$imageInfo	= getimagesize($this->imageFilepath)) exit();   // source image

        switch ($imageInfo[2]) {
            case IMAGETYPE_GIF :
                $imagecreate    = 'imagecreatefromgif';
                $imageformat    = 'imagegif';
                $imagequality   = null;
                $mimetype       = 'image/gif';
                break;
            case IMAGETYPE_JPEG :
                $imagecreate    = 'imagecreatefromjpeg';
                $imageformat    = 'imagejpeg';
                $imagequality   = $this->mg2->thumbquality;
                $mimetype       = 'image/jpeg';
                break;
            case IMAGETYPE_PNG :
                $imagecreate    = 'imagecreatefrompng';
                $imageformat    = 'imagepng';
                $imagequality   = min(floor($this->mg2->thumbquality/10), 9);
                $mimetype       = 'image/png';
                break;
            case IMAGETYPE_WEBP :
                $imagecreate    = 'imagecreatefromwebp';
                $imageformat    = 'imagewebp';
                $imagequality   = min(abs($this->mg2->thumbquality), 100);
                $mimetype       = 'image/webp';
                break;
            default: exit();
        }

        // IMAGE SIZE
        $imageWidth	 = max($imageInfo[0], 1);   // to avoid divison by zero
        $imageHeight = max($imageInfo[1], 1);   // to avoid divison by zero

        // RETURN VALUES
        return [
            $imageWidth,
            $imageHeight,
            $imagecreate,
            $imageformat,
            $imagequality,
            $mimetype,
        ];
    }

    /**
     * OUTPUT THUMBNAILS
     */
    public function outputThumbnail()
    {
        if (!$imageInfo = getimagesize($this->imageFilepath)) exit();

        // GET MIME TYPE
        $imageType = [
            IMAGETYPE_GIF  => 'image/gif',
            IMAGETYPE_JPEG => 'image/jpeg',
            IMAGETYPE_PNG  => 'image/png',
            IMAGETYPE_WEBP => 'image/webp',
        ];
        $mimeType = $imageType[$imageInfo[2]] ?? null;

        // OUTPUT IMAGE
        $this->outputHeader($mimeType);
        readfile($this->imageFilepath);
    }

    /**
     * @param string $mimeType
     */
    private function outputHeader($mimeType)
    {
        // DO NOT CACHE CONTENT
        header ('Expires: Mon, 22 Sep 1997 09:00:00 GMT');
        header ('Last-Modified: ' . date('r') . ' GMT');
        header ('Cache-Control: no-store, no-cache, must-revalidate');
        header ('Cache-Control: post-check=0, pre-check=0', false);
        header ('Pragma: no-cache');

        // CONTENT TYPE
        header('Content-type:'. $mimeType);
    }
}
